<?php
/**
 * @package   AkeebaLoginGuard
 * @copyright Copyright (c)2016-2022 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

namespace Akeeba\Component\LoginGuard\Administrator\Controller;

// Prevent direct access
defined('_JEXEC') or die;

use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\ControllerEvents;
use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\GetRedirectionAware;
use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\RegisterControllerTasks;
use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\ReturnURLAware;
use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\ReusableModels;
use Akeeba\Component\LoginGuard\Administrator\Event\NotifyActionLog;
use Akeeba\Component\LoginGuard\Administrator\Helper\Tfa as TfaHelper;
use Akeeba\Component\LoginGuard\Administrator\Model\MethodsModel;
use Exception;
use Joomla\CMS\Application\CMSApplication;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\User\UserFactoryInterface;
use Joomla\Input\Input;
use ReflectionObject;
use RuntimeException;

class MethodsController extends BaseController
{
	use ControllerEvents;
	use GetRedirectionAware;
	use RegisterControllerTasks;
	use ReturnURLAware;
	use ReusableModels;

	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		// We have to tell Joomla what is the name of the view, otherwise it defaults to the name of the *component*.
		$config['default_view'] = 'Methods';

		parent::__construct($config, $factory, $app, $input);
	}


	/**
	 * Disable Two Step Verification for the current user
	 *
	 * @param   bool   $cachable     Can this view be cached
	 * @param   array  $urlparams    An array of safe url parameters and their variable types, for valid values see
	 *                               {@link JFilterInput::clean()}.
	 *
	 * @return  self   The current JControllerLegacy object to support chaining.
	 */
	public function disable($cachable = false, $urlparams = [])
	{
		$this->assertLoggedInUser();

		$this->checkToken($this->input->getMethod());

		// Make sure I am allowed to edit the specified user
		$user_id = $this->input->getInt('user_id', null);
		$user    = ($user_id === null)
			? $this->app->getIdentity()
			: Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById($user_id);
		$user    = $user ?? Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById(0);

		if (!TfaHelper::canEditUser($user))
		{
			throw new RuntimeException(Text::_('JERROR_ALERTNOAUTHOR'), 403);
		}

		// Delete all TSV Methods for the user
		/** @var MethodsModel $model */
		$model   = $this->getModel('Methods', 'Administrator');
		$type    = null;
		$message = null;

		TfaHelper::triggerEvent(
			new NotifyActionLog('onComLoginguardControllerMethodsBeforeDisable', [$user])
		);

		try
		{
			$model->deleteAll($user);
		}
		catch (Exception $e)
		{
			$message = $e->getMessage();
			$type    = 'error';
		}

		// Redirect
		$url       = Route::_('index.php?option=com_loginguard&task=Methods.display&user_id=' . $user_id, false);
		$returnURL = $this->input->getBase64('returnurl');

		if (!empty($returnURL))
		{
			$url = base64_decode($returnURL);
		}

		$this->setRedirect($url, $message, $type);

		return $this;
	}

	/**
	 * List all available Two Step Validation Methods available and guide the user to setting them up
	 *
	 * @param   bool   $cachable     Can this view be cached
	 * @param   array  $urlparams    An array of safe url parameters and their variable types, for valid values see
	 *                               {@link JFilterInput::clean()}.
	 *
	 * @return  self   The current JControllerLegacy object to support chaining.
	 */
	public function display($cachable = false, $urlparams = [])
	{
		$this->assertLoggedInUser();
		$this->setSiteTemplateStyle();

		// Make sure I am allowed to edit the specified user
		$user_id = $this->input->getInt('user_id', null);
		$user    = ($user_id === null)
			? $this->app->getIdentity()
			: Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById($user_id);
		$user    = $user ?? Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById(0);

		if (!TfaHelper::canEditUser($user))
		{
			throw new RuntimeException(Text::_('JERROR_ALERTNOAUTHOR'), 403);
		}

		$returnURL  = $this->input->getBase64('returnurl');
		$viewLayout = $this->input->get('layout', 'default', 'string');
		$view       = $this->getView();
		$view->setLayout($viewLayout);
		$view->returnURL = $returnURL;
		$view->user      = $user;

		$backupCodesModel = $this->getModel('Backupcodes', 'Administrator');
		$view->setModel($backupCodesModel, false);

		parent::display($cachable, $urlparams);

		return $this;
	}

	/**
	 * Disable Two Step Verification for the current user
	 *
	 * @param   bool   $cachable     Can this view be cached
	 * @param   array  $urlparams    An array of safe url parameters and their variable types, for valid values see
	 *                               {@link JFilterInput::clean()}.
	 *
	 * @return  self   The current JControllerLegacy object to support chaining.
	 */
	public function dontshowthisagain($cachable = false, $urlparams = [])
	{
		$this->assertLoggedInUser();

		$this->checkToken($this->input->getMethod());

		// Make sure I am allowed to edit the specified user
		$user_id = $this->input->getInt('user_id', null);
		$user    = ($user_id === null)
			? $this->app->getIdentity()
			: Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById($user_id);
		$user    = $user ?? Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById(0);

		if (!TfaHelper::canEditUser($user))
		{
			throw new RuntimeException(Text::_('JERROR_ALERTNOAUTHOR'), 403);
		}

		TfaHelper::triggerEvent(
			new NotifyActionLog('onComLoginguardControllerMethodsBeforeDontshowthisagain', [$user])
		);

		/** @var MethodsModel $model */
		$model = $this->getModel('Methods', 'Administrator');
		$model->setFlag($user, true);

		// Redirect
		$url       = Uri::base();
		$returnURL = $this->input->getBase64('returnurl');

		if (!empty($returnURL))
		{
			$url = base64_decode($returnURL);
		}

		$this->setRedirect($url);

		return $this;
	}

	private function assertLoggedInUser()
	{
		$user = $this->app->getIdentity()
			?: Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById(0);

		if ($user->guest)
		{
			throw new RuntimeException(Text::_('JERROR_ALERTNOAUTHOR'), 403);
		}
	}

	/**
	 * Set a specific site template style in the frontend application
	 *
	 */
	private function setSiteTemplateStyle(): void
	{
		$Itemid = $this->app->input->get('Itemid');

		if (!empty($Itemid))
		{
			return;
		}

		$templateStyle = (int) ComponentHelper::getParams('com_loginguard')->get('captive_template', '');

		if (empty($templateStyle) || !$this->app->isClient('site'))
		{
			return;
		}

		$this->app->input->set('templateStyle', $templateStyle);

		$refApp      = new ReflectionObject($this->app);
		$refTemplate = $refApp->getProperty('template');
		$refTemplate->setAccessible(true);
		$refTemplate->setValue($this->app, null);

		$template = $this->app->getTemplate(true);

		$this->app->set('theme', $template->template);
		$this->app->set('themeParams', $template->params);
	}

}